==> haystack-8.x-1.0.tar.gz: haystack/src/Form/HaystackSettings.php <==
<?php

namespace Drupal\haystack\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use Drupal\haystack\HaystackCore;
use Symfony\Component\DependencyInjection\ContainerInterface;

class HaystackSettings extends FormBase {

  protected $haystack;
  protected $state;
  protected $dateFormatter;

  /**
   * HaystackSettings constructor.
   */
  public function __construct(
    HaystackCore $haystack,
    StateInterface $state,
    DateFormatterInterface $date_formatter
  ) {
    $this->haystack      = $haystack;
    $this->state         = $state;
    $this->dateFormatter = $date_formatter;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('haystack'),
      $container->get('state'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'haystack_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['haystack.settings'];
  }

  public function buildForm(
    array $form,
    \Drupal\Core\Form\FormStateInterface $form_state
  ) {
    $form                           = [];
    $form['#attached']['library'][] = 'haystack/styles';

    $health  = $this->haystack->healthCheck();
    $api_key = $this->haystack->getSetting('api_key');

    $form['original_api_key'] = [
      '#type'  => 'value',
      '#value' => $api_key,
    ];

    //API Key
    $form['api'] = [
      '#type'        => 'details',
      '#tree'        => FALSE,
      '#title'       => t('Server Settings'),
      '#open'        => empty($api_key) ? TRUE : FALSE,
      '#description' => t('Please enter your API key.'),
    ];

    $form['api']['api_key'] = [
      '#type'          => 'textfield',
      '#required'      => TRUE,
      '#title'         => t('API Key'),
      '#default_value' => $api_key,
    ];

    $form['api']['dev_mode'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Read-Only Mode'),
      '#description'   => t('If enabled no data will be send to the Haystack server.'),
      '#default_value' => $this->haystack->getSetting('dev_mode'),
    ];


    if ($health == 0) {
      //init
      $form['actions']['save_config'] = [
        '#type'          => 'submit',
        '#title'         => t('Save Configuration'),
        '#default_value' => t('Save Configuration'),
      ];

    }
    else {
      //ready to init
      $search_id_key              = $this->haystack->getSetting('search_id_key');
      $haystack_quick_links       = $this->haystack->getSetting('quick_links');
      $haystack_quick_links_title = $this->haystack->getSetting('quick_links_title');
      $haystack_status            = $this->haystack->searchStatus();


      $status = $this->t('Cron last ran: %time ago.',
        array('%time' => $this->dateFormatter->formatTimeDiffSince($this->state->get('system.cron_last'))));


      if ($health > 1) {
        $process_total = $this->haystack->getSetting('process_total',0);
        $process_pos = $this->haystack->getSetting('process_pos',0);
        
        //Search Update Status
        $form['status'] = [
          '#type'        => 'fieldset',
          '#tree'        => FALSE,
          '#title'       => t('Search Indexer Status'),
          '#collapsible' => TRUE,
          '#collapsed'   => FALSE,
        ];

        $haystack_total = $this->haystack->haystackTotalItems();

        $form['status']['#description'] = '<p>Your Haystack index contains '.$haystack_total.' items.';
        if ($process_total != 0) {
          $percent = round(($process_pos/$process_total) * 100);
          $form['status']['#description'] .= ' There are '.($process_total - $process_pos).' items left in the queue. Please note that the queue may be larger than the total of items depending on when you last ran Cron.';
        }
        $form['status']['#description'] .= '</p>';
      }

      //Id for Search Field
      $form['searchId'] = [
        '#type'        => 'details',
        '#tree'        => FALSE,
        '#title'       => t('Search ID Field'),
        '#open'        => FALSE,
        '#description' => t('The ID of the search field on your site (e.g. #search_id). If left blank the Haystack footer will be added to the website.'),
      ];

      $form['searchId']['search_id_key'] = [
        '#type'          => 'textfield',
        '#required'      => FALSE,
        '#title'         => t('Search ID Field'),
        '#default_value' => $search_id_key,
      ];

      //Links
      $form['ql'] = [
        '#type'        => 'details',
        '#tree'        => FALSE,
        '#title'       => t('Quick Links'),
        '#open'        => empty($haystack_quick_links) || empty($haystack_quick_links_title) ? TRUE : FALSE,
        '#description' => t('Please enter the top links and titles for your website. These will appear as the top links when Haystack is launched. If left empty the results will show up blank.'),
      ];

      $form['ql']['quick_links_title'] = [
        '#type'          => 'textfield',
        '#required'      => FALSE,
        '#title'         => t('Title for the quick links field.'),
        '#default_value' => $haystack_quick_links_title,
      ];

      $form['ql']['quick_links'] = [
        '#type'          => 'textarea',
        '#required'      => FALSE,
        '#title'         => t('<p>The top pages and links for your website in &lt;li&gt;&lt;a href="LINK"&gt;TITLE&lt;/a&gt;&lt;/li&gt; format.</p><p>The recommended number of list items is 5.</p>'),
        '#default_value' => $haystack_quick_links,
      ];

      //Content Types
      $form['ct'] = [
        '#type'  => 'details',
        '#tree'  => FALSE,
        '#title' => t('Content Types'),
        '#open'  => TRUE,
      ];

      //Types
      $types = $this->haystack->getContentTypes(TRUE);
      if ($health == 1) {
        $types_value = [];//array_keys($types); //All
      }
      else {
        $types_value = $this->haystack->getContentTypes();
      }
      $form['ct']['content_types'] = [
        '#type'          => 'checkboxes',
        '#required'      => FALSE,
        '#title'         => t('Include the following content types in the search index:'),
        '#default_value' => $types_value,
        '#options'       => $types,
      ];

      $form['menu'] = [
        '#type'  => 'details',
        '#tree'  => FALSE,
        '#title' => t('Menus'),
        '#open'  => TRUE,
      ];

      //Menus
      $menus = menu_ui_get_menus();
      if ($health == 1) {
        $menus_value = [];//array_keys($menus); //All
      }
      else {
        $menus_value = $this->haystack->getSetting('menus');
      }

      $form['menu']['menus'] = [
        '#type'          => 'checkboxes',
        '#required'      => FALSE,
        '#title'         => t('Include the following menus in the search index:'),
        '#default_value' => $menus_value,
        '#options'       => $menus,
      ];

      $form['actions']['save_config'] = [
        '#type'          => 'submit',
        '#title'         => t('Save Configuration'),
        '#default_value' => t('Save Configuration'),
      ];

      if ($health > 1) {
        $index_title                 = t('Re-Index Website');
        $form['actions']['re_index'] = [
          '#type'          => 'submit',
          '#title'         => $index_title,
          '#default_value' => $index_title,
          '#submit'        => ['::reindexSubmit']
        ];
      }
    }

    return $form;
  }

  public function validateForm(
    array &$form,
    \Drupal\Core\Form\FormStateInterface $form_state
  ) {
//    if ($form_state->getValue(['op']) == 'Save Configuration') {
//      _haystack_update_config($form, $form_state);
//       drupal_set_message('Haystack save config.');
//    }
//    elseif ($form_state->getValue(['op']) == 'Re-Index Website') {
//      drupal_set_message('Haystack rebuild');
//      _haystack_submit_reindex($form, $form_state);
//    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $ori_key = $form_state->getValue('original_api_key');
    $new_key = $form_state->getValue('api_key');
    $health  = $this->haystack->healthCheck();

    $original_types = array(
      'content_types' => $this->haystack->getSetting('content_types'),
      'menus'         => $this->haystack->getSetting('menus'),
    );

    $this->saveDefaultFields($form_state->cleanValues()->getValues());


    // We clear out the client link and token right away if there is something off with the API key.
    if (empty($ori_key) || $ori_key != $new_key) {
      $this->haystack->setSetting('haystack_client_hash', '');

      $success = $this->haystack->getCredentials($new_key);
      if (!$success) {
        $this->haystack->setSetting('api_key', '');
        drupal_set_message(t('The API Key you are using is not valid.'),
          'error');
        return;
      }

      //Else reprocess all content and send to the new index
      $package = array(
        'api_token' => $new_key,
      );
      $this->haystack->apiCall($package, 'index', 'delete_all');
      $this->haystack->setSetting('haystack_content_types', array());
      $this->haystack->setSetting('haystack_menus', array());

    }

    if ($health == 1) {
      $this->haystack->setSetting('first_index', TRUE);
    }
    if ($health > 0) {
      $this->processContentMenuChanges($form_state, $original_types);
    }

    drupal_set_message('Haystack configuration saved.');
  }

  public function reindexSubmit(array &$form, FormStateInterface $form_state) {
    $new_key = $form_state->getValue('api_key');

    //Delete all on server
    $package = array(
      'api_token' => $new_key,
    );
    $this->haystack->apiCall($package, 'index', 'delete_all');

    //Clear 
    $this->haystack->resetMeterTotal();

    $original_types = array(
      'content_types' => [],
      'menus'         => [],
    );

    //Process new contents against empty initial results
    $this->processContentMenuChanges($form_state, $original_types);
  }


  private function processContentMenuChanges(
    FormStateInterface $form_state,
    $original_types
  ) {



    $new_key = $form_state->getValue('api_key');

    //Check if content types have changed...
    $c_name = 'content_types';
    $m_name = 'menus';

    $types = array(
      $c_name => $form_state->getValue('content_types'),
      $m_name => $form_state->getValue('menus'),
    );


    $diff_add = array($c_name => array(), $m_name => array());
    $diff_sub = array($c_name => array(), $m_name => array());


    foreach ($types as $key => $val) { //Checks the difference and adds them, subrtract or add
      foreach ($val as $k => $v) {
        $ori = isset($original_types[$key][$k]) && $original_types[$key][$k] != '0';
        $new = $v != '0';

        if ($new && !$ori) {
          $diff_add[$key][$k] = $v;
        }
        elseif ($ori && !$new) {
          $diff_sub[$key][$k] = $v;
        }
      }
    }

    $package = array(
      'api_token' => $new_key,
    );


    //Delete Array
    foreach ($diff_sub as $key => $val) { //We use keys as the val may be set 0
      if (!empty($val)) {
        //Delete by Post Type
        if ($key == $c_name) { //Content
          foreach ($val as $k => $v) { //Delete each type individually
            $this->haystack->apiCall($package, $k, 'delete_type');
          }
        }

        //Delete by Menu Name
        if ($key == $m_name) {
          $this->haystack->indexMenus(array_keys($val),
            TRUE); //Delete is second param
        }
      }
    }

    //Add Array
    $tmp_total = 0;
    foreach ($diff_add as $key => $val) { //We use keys as the val may be set 0
      if (!empty($val)) {
        drupal_set_message(t('Haystack has added the following @key: @keys',
          array('@key' => $key, '@keys' => implode(', ', array_keys($val)))
        ));

        //Update by Post Type
        if ($key == $c_name) {
          foreach ($val as $k => $v) {
            $nids = $this->haystack->getNodes($k);
            $tmp_total += count($nids);

            foreach ($nids as $nid) {
              node_reindex_node_search($nid); // Mark node for reindexing
            }
          }
        }

        //Update by Menu Name
        elseif ($key == $m_name) {
          $this->haystack->indexMenus(array_keys($val)); //Reindex menu
        }
      }      
    }

    $this->haystack->addMeterTotal($tmp_total);
  }

  private function saveDefaultFields($values) {
    foreach ($values as $key => $field) {
      $this->haystack->setSetting($key, $field);
    }
  }


}

==> haystack-8.x-1.1.tar.gz: haystack/src/Form/HaystackSettings.php <==
<?php

namespace Drupal\haystack\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use Drupal\haystack\HaystackCore;
use Symfony\Component\DependencyInjection\ContainerInterface;

class HaystackSettings extends FormBase {

  protected $haystack;
  protected $state;
  protected $dateFormatter;

  /**
   * HaystackSettings constructor.
   */
  public function __construct(
    HaystackCore $haystack,
    StateInterface $state,
    DateFormatterInterface $date_formatter
  ) {
    $this->haystack      = $haystack;
    $this->state         = $state;
    $this->dateFormatter = $date_formatter;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('haystack'),
      $container->get('state'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'haystack_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['haystack.settings'];
  }

  public function buildForm(
    array $form,
    \Drupal\Core\Form\FormStateInterface $form_state
  ) {
    $form                           = [];
    $form['#attached']['library'][] = 'haystack/styles';

    $health  = $this->haystack->healthCheck();
    $api_key = $this->haystack->getSetting('api_key');

    $form['original_api_key'] = [
      '#type'  => 'value',
      '#value' => $api_key,
    ];

    //API Key
    $form['api'] = [
      '#type'        => 'details',
      '#tree'        => FALSE,
      '#title'       => t('Server Settings'),
      '#open'        => empty($api_key) ? TRUE : FALSE,
      '#description' => t('Please enter your API key.'),
    ];

    $form['api']['api_key'] = [
      '#type'          => 'textfield',
      '#required'      => TRUE,
      '#title'         => t('API Key'),
      '#default_value' => $api_key,
    ];

    $form['api']['dev_mode'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Read-Only Mode'),
      '#description'   => t('If enabled no data will be send to the Haystack server.'),
      '#default_value' => $this->haystack->getSetting('dev_mode'),
    ];


    if ($health == 0) {
      //init
      $form['actions']['save_config'] = [
        '#type'          => 'submit',
        '#title'         => t('Save Configuration'),
        '#default_value' => t('Save Configuration'),
      ];

    }
    else {
      //ready to init
      $search_id_key              = $this->haystack->getSetting('search_id_key');
      $haystack_quick_links       = $this->haystack->getSetting('quick_links');
      $haystack_quick_links_title = $this->haystack->getSetting('quick_links_title');
      $haystack_status            = $this->haystack->searchStatus();


      $status = $this->t('Cron last ran: %time ago.',
        array('%time' => $this->dateFormatter->formatTimeDiffSince($this->state->get('system.cron_last'))));


      if ($health > 1) {
        $process_total = $this->haystack->getSetting('process_total',0);
        $process_pos = $this->haystack->getSetting('process_pos',0);
        
        //Search Update Status
        $form['status'] = [
          '#type'        => 'fieldset',
          '#tree'        => FALSE,
          '#title'       => t('Search Indexer Status'),
          '#collapsible' => TRUE,
          '#collapsed'   => FALSE,
        ];

        $haystack_total = $this->haystack->haystackTotalItems();

        $form['status']['#description'] = '<p>Your Haystack index contains '.$haystack_total.' items.';
        if ($process_total != 0) {
          $percent = round(($process_pos/$process_total) * 100);
          $form['status']['#description'] .= ' There are '.($process_total - $process_pos).' items left in the queue. Please note that the queue may be larger than the total of items depending on when you last ran Cron.';
        }
        $form['status']['#description'] .= '</p>';
      }

      //Id for Search Field
      $form['searchId'] = [
        '#type'        => 'details',
        '#tree'        => FALSE,
        '#title'       => t('Search ID Field'),
        '#open'        => FALSE,
        '#description' => t('The ID of the search field on your site (e.g. #search_id). If left blank the Haystack footer will be added to the website.'),
      ];

      $form['searchId']['search_id_key'] = [
        '#type'          => 'textfield',
        '#required'      => FALSE,
        '#title'         => t('Search ID Field'),
        '#default_value' => $search_id_key,
      ];

      //Links
      $form['ql'] = [
        '#type'        => 'details',
        '#tree'        => FALSE,
        '#title'       => t('Quick Links'),
        '#open'        => empty($haystack_quick_links) || empty($haystack_quick_links_title) ? TRUE : FALSE,
        '#description' => t('Please enter the top links and titles for your website. These will appear as the top links when Haystack is launched. If left empty the results will show up blank.'),
      ];

      $form['ql']['quick_links_title'] = [
        '#type'          => 'textfield',
        '#required'      => FALSE,
        '#title'         => t('Title for the quick links field.'),
        '#default_value' => $haystack_quick_links_title,
      ];

      $form['ql']['quick_links'] = [
        '#type'          => 'textarea',
        '#required'      => FALSE,
        '#title'         => t('<p>The top pages and links for your website in &lt;li&gt;&lt;a href="LINK"&gt;TITLE&lt;/a&gt;&lt;/li&gt; format.</p><p>The recommended number of list items is 5.</p>'),
        '#default_value' => $haystack_quick_links,
      ];

      //Content Types
      $form['ct'] = [
        '#type'  => 'details',
        '#tree'  => FALSE,
        '#title' => t('Content Types'),
        '#open'  => TRUE,
      ];

      //Types
      $types = $this->haystack->getContentTypes(TRUE);
      if ($health == 1) {
        $types_value = [];//array_keys($types); //All
      }
      else {
        $types_value = $this->haystack->getContentTypes();
      }
      $form['ct']['content_types'] = [
        '#type'          => 'checkboxes',
        '#required'      => FALSE,
        '#title'         => t('Include the following content types in the search index:'),
        '#default_value' => $types_value,
        '#options'       => $types,
      ];

      $form['menu'] = [
        '#type'  => 'details',
        '#tree'  => FALSE,
        '#title' => t('Menus'),
        '#open'  => TRUE,
      ];

      //Menus
      $menus = menu_ui_get_menus();
      if ($health == 1) {
        $menus_value = [];//array_keys($menus); //All
      }
      else {
        $menus_value = $this->haystack->getSetting('menus');
      }

      $form['menu']['menus'] = [
        '#type'          => 'checkboxes',
        '#required'      => FALSE,
        '#title'         => t('Include the following menus in the search index:'),
        '#default_value' => $menus_value,
        '#options'       => $menus,
      ];

      $form['actions']['save_config'] = [
        '#type'          => 'submit',
        '#title'         => t('Save Configuration'),
        '#default_value' => t('Save Configuration'),
      ];

      if ($health > 1) {
        $index_title                 = t('Re-Index Website');
        $form['actions']['re_index'] = [
          '#type'          => 'submit',
          '#title'         => $index_title,
          '#default_value' => $index_title,
          '#submit'        => ['::reindexSubmit']
        ];
      }
    }

    return $form;
  }

  public function validateForm(
    array &$form,
    \Drupal\Core\Form\FormStateInterface $form_state
  ) {
//    if ($form_state->getValue(['op']) == 'Save Configuration') {
//      _haystack_update_config($form, $form_state);
//       drupal_set_message('Haystack save config.');
//    }
//    elseif ($form_state->getValue(['op']) == 'Re-Index Website') {
//      drupal_set_message('Haystack rebuild');
//      _haystack_submit_reindex($form, $form_state);
//    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $ori_key = $form_state->getValue('original_api_key');
    $new_key = $form_state->getValue('api_key');
    $health  = $this->haystack->healthCheck();

    $original_types = array(
      'content_types' => $this->haystack->getSetting('content_types'),
      'menus'         => $this->haystack->getSetting('menus'),
    );

    $this->saveDefaultFields($form_state->cleanValues()->getValues());


    // We clear out the client link and token right away if there is something off with the API key.
    if (empty($ori_key) || $ori_key != $new_key) {
      $this->haystack->setSetting('client_hash', '');

      $success = $this->haystack->getCredentials($new_key);
      if (!$success) {
        $this->haystack->setSetting('api_key', '');
        drupal_set_message(t('The API Key you are using is not valid.'),
          'error');
        return;
      }

      //Else reprocess all content and send to the new index
      $package = array(
        'api_token' => $new_key,
      );
      $this->haystack->apiCall($package, 'index', 'delete_all');
      $this->haystack->setSetting('content_types', array());
      $this->haystack->setSetting('menus', array());

    }

    if ($health == 1) {
      $this->haystack->setSetting('first_index', TRUE);
    }
    if ($health > 0) {
      $this->processContentMenuChanges($form_state, $original_types);
    }

    drupal_set_message('Haystack configuration saved.');
  }

  public function reindexSubmit(array &$form, FormStateInterface $form_state) {
    $new_key = $form_state->getValue('api_key');

    //Delete all on server
    $package = array(
      'api_token' => $new_key,
    );
    $this->haystack->apiCall($package, 'index', 'delete_all');

    //Clear 
    $this->haystack->resetMeterTotal();

    $original_types = array(
      'content_types' => [],
      'menus'         => [],
    );

    //Process new contents against empty initial results
    $this->processContentMenuChanges($form_state, $original_types);
  }


  private function processContentMenuChanges(
    FormStateInterface $form_state,
    $original_types
  ) {



    $new_key = $form_state->getValue('api_key');

    //Check if content types have changed...
    $c_name = 'content_types';
    $m_name = 'menus';

    $types = array(
      $c_name => $form_state->getValue('content_types'),
      $m_name => $form_state->getValue('menus'),
    );


    $diff_add = array($c_name => array(), $m_name => array());
    $diff_sub = array($c_name => array(), $m_name => array());


    foreach ($types as $key => $val) { //Checks the difference and adds them, subrtract or add
      foreach ($val as $k => $v) {
        $ori = isset($original_types[$key][$k]) && $original_types[$key][$k] != '0';
        $new = $v != '0';

        if ($new && !$ori) {
          $diff_add[$key][$k] = $v;
        }
        elseif ($ori && !$new) {
          $diff_sub[$key][$k] = $v;
        }
      }
    }

    $package = array(
      'api_token' => $new_key,
    );


    //Delete Array
    foreach ($diff_sub as $key => $val) { //We use keys as the val may be set 0
      if (!empty($val)) {
        //Delete by Post Type
        if ($key == $c_name) { //Content
          foreach ($val as $k => $v) { //Delete each type individually
            $this->haystack->apiCall($package, $k, 'delete_type');
          }
        }

        //Delete by Menu Name
        if ($key == $m_name) {
          $this->haystack->indexMenus(array_keys($val),
            TRUE); //Delete is second param
        }
      }
    }

    //Add Array
    $tmp_total = 0;
    foreach ($diff_add as $key => $val) { //We use keys as the val may be set 0
      if (!empty($val)) {
        drupal_set_message(t('Haystack has added the following @key: @keys',
          array('@key' => $key, '@keys' => implode(', ', array_keys($val)))
        ));

        //Update by Post Type
        if ($key == $c_name) {
          foreach ($val as $k => $v) {
            $nids = $this->haystack->getNodes($k);
            $tmp_total += count($nids);

            foreach ($nids as $nid) {
              node_reindex_node_search($nid); // Mark node for reindexing
            }
          }
        }

        //Update by Menu Name
        elseif ($key == $m_name) {
          $this->haystack->indexMenus(array_keys($val)); //Reindex menu
        }
      }      
    }

    $this->haystack->addMeterTotal($tmp_total);
  }

  private function saveDefaultFields($values) {
    foreach ($values as $key => $field) {
      $this->haystack->setSetting($key, $field);
    }
  }


}
